local TICRATE = TICRATE

local function HasPlayerFinished(p)
	return (p.exiting > 0 or p.pflags & PF_TIMEOVER)
end

local function DD_finished(p)
	if p.spectator or (not p.mo) then
		return true
	elseif p.pflags & PF_TIMEOVER then
		return true
	elseif p.exiting then
		return true
	end
	return false
end

local function DD_allfinished()
	for p in players.iterate
		if not DD_finished(p) then
			return false
		end
	end
	return true
end

local function DD_ClearRank(p)
	local finishers = {}
	local finalposition = p.kartstuff[k_position]
	
	for t in players.iterate do
		if (t.exiting > 0 and not(t.pflags & PF_TIMEOVER)) and (t.mo and t.mo.valid) then
			table.insert(finishers, t)
		end
	end
		
	table.sort(finishers, function(a,b)
		return a.realtime < b.realtime
	end)
		
	-- Now... find their position in the table
	for i=1, #finishers do
		if(#finishers[i] == #p) then
			finalposition = i
			--print(finalposition)
			break
		end
	end
	
	if finalposition > 1 then
		local speeddiff = p.realtime - finishers[1].realtime 
		--print(speeddiff)

		for i = 1, speeddiff
			if i%TICRATE/7 == 0 then
				if p.DDspeedscore then
					p.DDspeedscore = max($-4, 0)
					--print("You're too Slow! Speed Score is now "..p.DDspeedscore)
				end
			end
		end
	end
end

addHook("MusicChange", function(old, new, mflags, looping, position, prefadems, fadems)
	if not server.XITdoomsday then return end
	
	old = $:upper()
	new = $:upper()
	
	if old == "DDSHIP" and new ~= "DDSHIP" then
		return true
	end
end)

local function DD_ScoreInit(p)
	p.DDspeedscore = 600
	p.DDdrivescore = 400
end

local function DD_ClearRankHandler()
	if not server.XITdoomsday then return end

	for p in players.iterate
		if not p.ddscoreinit then
			DD_ScoreInit(p)
			p.ddscoreinit = true
		end

		if HasPlayerFinished(p)
		and not p.ddayrank
			if not p.ddayeval
				if (p.pflags & PF_TIMEOVER) then
					S_StartSound(p.mo, sfx_ddfail, p)
					p.ddayfail = true
					p.ddayeval = true
				else
					S_StartSound(p.mo, sfx_ddpass, p)

					if not p.ddscorecalc then
						DD_ClearRank(p)
						p.ddscorecalc = true
					end

					p.ddayeval = true
				end
			end
			
			if (p.ddayeval and not p.ddayfail) then
				if not (S_SoundPlaying(p.mo, sfx_ddpass)
				or S_SoundPlaying(p.mo, sfx_ddclr1)
				or S_SoundPlaying(p.mo, sfx_ddclr2)
				or S_SoundPlaying(p.mo, sfx_ddclr3)
				or S_SoundPlaying(p.mo, sfx_ddclr4))
				and p.ddayranktimer < 122 then
					
					S_StopMusic(p)
					
					p.ddfinalscore = p.DDspeedscore + p.DDdrivescore

					if p.ddfinalscore >= 900 then
						S_StartSound(p.mo, sfx_ddclr1, p)
						p.ddclearrank = "\\c47S" 
					elseif p.ddfinalscore >= 750 then
						S_StartSound(p.mo, sfx_ddclr2, p)
						p.ddclearrank = "\\c22A"
					elseif p.ddfinalscore >= 600 then
						S_StartSound(p.mo, sfx_ddclr3, p)
						p.ddclearrank = "\\c73B"
					else
						S_StartSound(p.mo, sfx_ddclr4, p)
						p.ddclearrank = "\\c98C"
					end
					if DD_allfinished() then
						K_SetExitCountdown(10*TICRATE)
					end
				end
			end
			
			if not p.ddayfail then
				if not S_SoundPlaying(p.mo, sfx_ddpass)
					p.ddayranktimer = min($ + 1, 122)
				end
				local message
				if p.ddayranktimer == 1 then
					message = "\\c76PONTOS DE VELOCIDADE: \\c42"..p.DDspeedscore.."p\\c76/600p"
					message = DoomsdayUtils.MessageParser($)
					DoomsdayUtils.AddTooltip(p,message,nil)
				elseif p.ddayranktimer == 84 then
					message = "\n\\c76PONTOS DE DIRECAO: \\c42"..p.DDdrivescore.."p\\c76/400p"
					message = DoomsdayUtils.MessageParser($)
					DoomsdayUtils.UpdateTooltip(p,1,message,true)
				elseif p.ddayranktimer == 121 then
					message = "\\c76PONTOS FINAIS DA MISSAO: \\c42"..p.ddfinalscore.."p\\c76/1000p\nRanking: "..p.ddclearrank
					message = DoomsdayUtils.MessageParser($)
					DoomsdayUtils.UpdateTooltip(p,1,message,false)
				end
				
				if not (S_SoundPlaying(p.mo, sfx_ddclr1)
				or S_SoundPlaying(p.mo, sfx_ddclr2)
				or S_SoundPlaying(p.mo, sfx_ddclr3)
				or S_SoundPlaying(p.mo, sfx_ddclr4))
				and p.ddayranktimer == 122	
					S_ChangeMusic("DDSHIP", true, p)
					p.ddayrank = true
				end
			end
		end
	end
end

addHook("ThinkFrame", DD_ClearRankHandler)

---Penalties Go Here

local function DD_Penalty(p, num)
	if not p.DDdrivescore then return end
	
	p.DDdrivescore = max($ - num, 0)
end

local function DD_MiscPenalties(p)
	if not server.XITdoomsday then return end
	if (not p.DDdrivescore) or p.exiting then return end
	if p.spectator then return end
	
	local pks = p.kartstuff

	if pks[k_offroad] > (FRACUNIT/2)
	and not (pks[k_hyudorotimer] > 0
	or pks[k_invincibilitytimer] > 0
	or pks[k_sneakertimer] > 0
	or (HugeQuest and p.hugequest.huge > 0 and hq_offroad.value)) then
		p.DDoffroad = $ + 1
		
		if p.DDoffroad%(TICRATE/5) == 0 then
			DD_Penalty(p, 3)
		end
		--print("Offroad timer: ".. pks[k_offroad])
	else
		p.DDoffroad = 0
	end
	
	if p.mo.eflags & MFE_JUSTBOUNCEDWALL
	and not (pks[k_spinouttimer] > 0 or pks[k_invincibilitytimer] > 0 or (pks[k_growshrinktimer] > 0 or (HugeQuest and p.hugequest.huge > 0)) or pks[k_hyudorotimer] > 0 or p.powers[pw_flashing] > 0) then
		DD_Penalty(p, 20)
		--print("Wall Hit, Drive Score now "..p.DDdrivescore)
	end
	
	if p.DDrespawn then
		DD_Penalty(p, 60)
		--print("You died, Drive Score now "..p.DDdrivescore)
		p.DDrespawn = nil
	end
end
addHook("PlayerThink", DD_MiscPenalties)


local function DD_Respawn(mo)
	if not server.XITdoomsday then return end
	if not (mo and mo.valid and mo.player) then return end
	
	if not (mo.player.pflags & PF_TIMEOVER) then
		mo.player.DDrespawn = true
	end
end
addHook("MobjDeath", DD_Respawn, MT_PLAYER)

addHook("PlayerSpin", function(p, inf, s)
	if not server.XITdoomsday then return end
	if (not p.DDdrivescore) or p.exiting then return end

	if inf and inf.valid then
		DD_Penalty(p, 40)
		--print("You didn't block, Drive Score now "..p.DDdrivescore)
	end
end)

addHook("PlayerSquish", function(p, inf, s)
	if not server.XITdoomsday then return end
	if (not p.DDdrivescore) or p.exiting then return end
	
	if inf and inf.valid then
		DD_Penalty(p, 30)
		--print("Welcome to IHOP, Drive Score now "..p.DDdrivescore)
	end
end)

addHook("PlayerExplode", function(p, inf, s)
	if not server.XITdoomsday then return end
	if (not p.DDdrivescore) or p.exiting then return end
	
	if inf and inf.valid then
		DD_Penalty(p, 60)
		--print("You blew up, Drive Score now "..p.DDdrivescore)
	end
end)

addHook("MapLoad", function()
	for p in players.iterate
		p.ddayeval = nil
		p.ddayfail = nil
		p.ddayrank = nil
		p.ddclearrank = nil
		p.ddscorecalc = nil
		p.ddfinalscore = 0
		p.ddscoreinit = nil
	end
end)

addHook("PlayerSpawn", function(p)
	p.ddayranktimer = 0
	p.DDoffroad = 0
end)