//Perlin Noise Randomness

// LJ Perlin

rawset(_G, "perlin", {})

local x, y, z, w

function perlin.randomNumber()
    local t = x ^^ (x << 11)
    x, y, z, w = y, z, w, w ^^ (w >> 19) ^^ t ^^ (t >> 8)
    return w
end

function perlin.randomRange(a, b)
    return a + abs(perlin.randomNumber() % (b - a))
end

function perlin.randomChance(chance)
    return perlin.randomRange(0, FRACUNIT) <= chance
end

function perlin.setSeed(seed)
    x, y, z, w = seed, 3154710, 9406548, 1028369
    for _ = 1, perlin.randomRange(10, 100)
        perlin.randomNumber()
    end
end

perlin.setSeed(5197528)

addHook("NetVars", function(n)
    x = n($)
    y = n($)
    z = n($)
    w = n($)
end)

//Tooltip Queue

freeslot("sfx_ddtxt1","sfx_ddtxt2")
rawset(_G, "DoomsdayUtils", {})
rawset(_G, "TooltipQueue", {})

addHook("NetVars", function(net)
	TooltipQueue = net($)
end)

addHook("MapChange", function(map)
	TooltipQueue = {}
end)

DoomsdayUtils.MessageParser = function(m)
	local message = m
	//adds new line characters for commands to use
	message = string.gsub($, "\\n", "\n")
	//convert custom color codes into escape sequences 128-227
	for i = 100,1,-1 do
		message = string.gsub($, "\\c"..i, string.char(i+127))
	end
	
	message = string.gsub($, "<\\wave>", "\228")
	message = string.gsub($, "<\\shake>", "\229")
	message = string.gsub($, "<\\fade>", "\230")
	message = string.gsub($, "<\\flash>", "\231")
	message = string.gsub($, "<\\rainbow>", "\232")
	
	message = string.gsub($, "<\\reset>", "\255")
	return message
end

/**
 * p - Player
 * m - Message Being Sent
 * t - time limit of message (set to -1 or nil to remain on screen until next message)
 */
DoomsdayUtils.AddTooltip = function(p, m, t)
	if not p or not p.valid then return end
	if not m then return end
	m = DoomsdayUtils.MessageParser($)
	t = tonumber(t)
	if not t or t <= 0 then t = -1 end
	if not TooltipQueue then
		TooltipQueue = {}
	end
	if not TooltipQueue[#p] then
		TooltipQueue[#p] = {}		
	end
	local tooltip = {}
	tooltip.style = "doomsday"
	tooltip.message = m
	tooltip.max_time = t
	tooltip.live_time = 0
	tooltip.trans_time = 0
	table.insert(TooltipQueue[#p], tooltip)
end

DoomsdayUtils.UpdateTooltip = function(p, index, m, append)
	if not p or not p.valid then return end
	if not m then return end
	m = DoomsdayUtils.MessageParser($)
	local queue = TooltipQueue[#p]
	if not queue[index] then return end
	
	if append then
		queue[index].message = $ .. m
	else
		queue[index].message = m
	end
end 

COM_AddCommand("doomsday_say", function(p, m, t)
	if not t then t = 3 * TICRATE end
	for player in players.iterate
		DoomsdayUtils.AddTooltip(player, m, t)
	end
end, COM_ADMIN)

local max_trans_time = 9
addHook("PlayerThink", function(p)
	if not TooltipQueue then return end
	if not TooltipQueue[#p] then return end
	if not TooltipQueue[#p][1] then return end
	local tooltip = TooltipQueue[#p][1]
	if not tooltip.style or tooltip.style != "doomsday" then return end
	
	if TooltipQueue[#p][2] and tooltip.max_time == -1 then
		tooltip.max_time = 2 * TICRATE
	end
	
	if tooltip.trans_time < max_trans_time and tooltip.live_time == 0 then
		tooltip.trans_time = $ + 1
		if tooltip.trans_time == 1 then
			S_StartSound(p.mo, sfx_ddtxt1, p)
		end
	elseif tooltip.trans_time == max_trans_time and tooltip.live_time < tooltip.max_time then
		tooltip.live_time = $ + 1
	elseif tooltip.trans_time > 0 and tooltip.live_time == tooltip.max_time
		tooltip.trans_time = $ - 1
		if tooltip.trans_time == max_trans_time - 1 then
			S_StartSound(p.mo, sfx_ddtxt2, p)
		end
	end
	
	if tooltip.trans_time == 0 and tooltip.live_time > 0 then
		table.remove(TooltipQueue[#p], 1)
	end
end)

local escape_width = {}
for i = 128,256 do
	escape_width[i] = 0
end

FNT_NewFont("FONT_DOOMSDAY",{prefix = "DOMFN%03d",spacewidth = 4, forcedwidths = escape_width})

local EF_WAVE = 1
local EF_SHAKE = 2
local EF_FADE = 4
local EF_FLASH = 8
local EF_RAINBOW = 16

local function colorParser(drawdata)
	if not drawdata.effects then
		drawdata.effects = 0
	end
	if not drawdata.oldcolor then
		drawdata.oldcolor = 0
	end
	
	if not drawdata.oldflags then
		drawdata.oldflags = 0
	end
	drawdata.xoff = 0
	drawdata.yoff = 0
	if drawdata.curascii >= 128 and drawdata.curascii <= 227 then
		drawdata.color = drawdata.curascii - 127
		drawdata.oldcolor = drawdata.color
	end
	
	// - sin wave for 228
	if drawdata.curascii == 228 then
		if drawdata.effects & EF_WAVE then
			drawdata.effects = $ & ~EF_WAVE
		else
			drawdata.effects = $ | EF_WAVE
		end
	end
	if drawdata.effects & EF_WAVE then
		local speed = 8
		local range = 2
		local dissonance = 10
		drawdata.yoff = $ + sin(FixedAngle((leveltime+(dissonance / 2)*drawdata.curchar)*FRACUNIT*speed)) * range
	end
	
	// - shaking for 229
	if drawdata.curascii == 229 then
		if drawdata.effects & EF_SHAKE then
			drawdata.effects = $ & ~EF_SHAKE
		else
			drawdata.effects = $ | EF_SHAKE
		end
	end
	if drawdata.effects & EF_SHAKE then
		drawdata.xoff = $ + perlin.randomRange(-2,2) * FRACUNIT / 2
		drawdata.yoff = $ + perlin.randomRange(-2,2) * FRACUNIT / 2
	end
	
	// - fade in and out for 230
	if drawdata.curascii == 230 then
		if drawdata.effects & EF_FADE then
			drawdata.effects = $ & ~EF_FADE
			drawdata.flags = drawdata.oldflags
		else
			drawdata.effects = $ | EF_FADE
			drawdata.oldflags = drawdata.flags
		end
	end
	if drawdata.effects & EF_FADE then
		drawdata.flags = $ | ((FixedInt(sin(leveltime * ANG10) * 4)+5) << V_ALPHASHIFT)
	end
	
	// - flash orange for 231
	if drawdata.curascii == 231 then
		if drawdata.effects & EF_FLASH then
			drawdata.effects = $ & ~EF_FLASH
			drawdata.color = drawdata.oldcolor
		else
			drawdata.effects = $ | EF_FLASH
			drawdata.oldcolor = drawdata.color
		end
	end
	if drawdata.effects & EF_FLASH then
		drawdata.color = FixedInt(sin(leveltime * ANG30) * 2)+35
	end
	
	// - rainbow for 232
	if drawdata.curascii == 232 then
		if drawdata.effects & EF_RAINBOW then
			drawdata.effects = $ & ~EF_RAINBOW
			drawdata.color = drawdata.oldcolor
		else
			drawdata.effects = $ | EF_RAINBOW
			drawdata.oldcolor = drawdata.color
		end
	end
	if drawdata.effects & EF_RAINBOW then
		local rainbow = {24, 34, 35, 47, 38, 61, 63, 72, 74, 85, 86, 93, 98, 94}
		
		drawdata.color = rainbow[(((leveltime / 4) + drawdata.curchar) % #rainbow)+1]
	end
	
	// - reset for all movement
	if drawdata.curascii == 255 then
		drawdata.effects = 0
	end
end

hud.add(function(v,p,c)
	if not TooltipQueue then return end
	if not TooltipQueue[#p] then return end
	if not TooltipQueue[#p][1] then return end
	local tooltip = TooltipQueue[#p][1]
	if not tooltip.style or tooltip.style != "doomsday" then return end
	local trans_flag = V_10TRANS
	
	if tooltip.trans_time < max_trans_time - 1 then
		trans_flag = (max_trans_time  - tooltip.trans_time) << V_ALPHASHIFT
	end
	
	local gfx_prefix = "DDAY"
	local color = SKINCOLOR_JAWZ
	if server.XITdoomsdaychange then
		color = SKINCOLOR_YELLOW
	end
	if server.XITdoomsday == "Disaster2" then
		color = SKINCOLOR_CRIMSON
	end

	local cachedColor = v.getColormap(TC_DEFAULT, color)
	
	if not splitscreen then
		if tooltip.trans_time then
			v.drawScaled(160*FRACUNIT,142*FRACUNIT,8*FRACUNIT, v.cachePatch(gfx_prefix.."TEXM"), V_SNAPTOBOTTOM|trans_flag, cachedColor)
			v.drawScaled(0,140*FRACUNIT,FRACUNIT, v.cachePatch(gfx_prefix.."TEXL"),V_SNAPTOBOTTOM|V_SNAPTOLEFT|trans_flag, cachedColor)
			v.drawScaled(224*FRACUNIT,140*FRACUNIT,FRACUNIT, v.cachePatch(gfx_prefix.."TEXR"),V_SNAPTOBOTTOM|V_SNAPTORIGHT|trans_flag, cachedColor)
			FNT_Write(v, {x = 160 * FRACUNIT, y = 154*FRACUNIT, text = tooltip.message,font = FONT_DOOMSDAY, scale = FRACUNIT, color = SKINCOLOR_WHITE, halign = HALIGN_CENTER, valign = VALIGN_CENTER, flags = V_SNAPTOBOTTOM|trans_flag, predrawfunc = colorParser})
		end
	--TODO: ADD SPLITSCREEN SUPPORT THAT WORKS
	end
end)