-- ROWKET LAWNCHAIR, by RetroStation

local TICRATE = TICRATE
local FRACUNIT = FRACUNIT
local ANG1 = ANG1
local k_itemamount = k_itemamount
local k_spinouttimer = k_spinouttimer

--item flags
local XIF_POWERITEM = 1 --is power item (affects final odds)
local XIF_COOLDOWNONSTART = 2 --can't be obtained on start cooldown
local XIF_UNIQUE = 4 --only one can exist in anyone's slot
local XIF_LOCKONUSE = 8 --locks the item slot when the item is used, slot must be unlocked manually by setting player.xItemData.xItem_itemSlotLocked to false
local XIF_COOLDOWNINDIRECT = 16 --checks if indirectitemcooldown is 0
local XIF_COLPATCH2PLAYER = 32 --map hud patch colour to player prefcolor
local XIF_ICONFORAMT = 64 --item icon changes depending on the item amount (animation frames become amount frames)


freeslot(
	"sfx_rlget",
	"sfx_rlshot",
	"MT_LAUNCHERROCKET",
	"SPR_XRCK",
	"SPR_XRKT",
	"S_LAUNCHERROCKET",
	"S_ROCKETDED",
	"MT_VERTLROCKET",
	"S_VERTLROCKET",
	"SPR_VRKT"
)

function A_BurstRing(actor, var1, var2)
	local radius = 3*(FixedMul((3 * (192 * FRACUNIT)) / 2, mapobjectscale) + (32 * mapobjectscale))/4
	for i = 0, 16 do
		local dust = P_SpawnMobj(actor.x, actor.y, actor.z, MT_SMOKE)
		dust.state = S_OPAQUESMOKE1
		dust.angle = (ANGLE_180/8) * i
		dust.scale = actor.scale
		dust.destscale = actor.scale*10
		dust.scalespeed = actor.scale/12
		P_InstaThrust(dust, dust.angle, FixedMul(16*FRACUNIT, actor.scale))
		
		K_SpawnKartExplosion(actor.x, actor.y, actor.z,
				radius, 32, MT_BOOSTFLAME,
				i * (ANGLE_45 / 4), true, true, actor)
				
		K_SpawnKartExplosion(actor.x, actor.y, actor.z,
				radius, 32, MT_MINEEXPLOSION,
				i * (ANGLE_45 / 4), true, false, actor)
	end
	P_SpawnMobj(actor.x, actor.y, actor.z, MT_MINEEXPLOSIONSOUND)
end

states[S_VERTLROCKET] = {SPR_VRKT, FF_ANIMATE|A, -1, nil, 5, 2, S_VERTLROCKET}

states[S_LAUNCHERROCKET] = {SPR_XRKT, FF_ANIMATE|A, -1, nil, 5, 2, S_LAUNCHERROCKET}

states[S_ROCKETDED] = {
	sprite = SPR_NULL,
	frame = A,
	action = A_BurstRing,
	tics = TICRATE,
	nextstate = S_NULL
}

mobjinfo[MT_LAUNCHERROCKET] = {
	doomednum = -1,             // doomednum
	spawnstate = S_LAUNCHERROCKET,        // spawnstate
	spawnhealth = 1,              // spawnhealth
	seestate = S_NULL,         // seestate
	seesound = sfx_rlshot,     // seesound
	reactiontime = 8,              // reactiontime
	attacksound = sfx_None,       // attacksound
	painchance = 64*FRACUNIT,              // painchance
	painsound = sfx_None,       // painsound
	deathstate = S_ROCKETDED,   // deathstate
	deathsound = sfx_s3k5d,      // deathsound
	speed = 64*FRACUNIT,    // speed
	radius = 16*FRACUNIT,    // radius
	height = 32*FRACUNIT,    // height
	dispoffset = 0,              // display offset
	mass = 100,            // mass
	damage = 1,              // damage
	activesound = sfx_s3kc0s,     // activesound
	flags = MF_SHOOTABLE|MF_MISSILE|MF_DONTENCOREMAP  // flags
}

mobjinfo[MT_VERTLROCKET] = {
	doomednum = -1,             // doomednum
	spawnstate = S_VERTLROCKET,        // spawnstate
	spawnhealth = 1,              // spawnhealth
	seestate = S_NULL,         // seestate
	seesound = sfx_rlshot,     // seesound
	reactiontime = 8,              // reactiontime
	attacksound = sfx_None,       // attacksound
	painchance = 64*FRACUNIT,              // painchance
	painsound = sfx_None,       // painsound
	deathstate = S_ROCKETDED,   // deathstate
	deathsound = sfx_s3k5d,      // deathsound
	speed = 64*FRACUNIT,    // speed
	radius = 16*FRACUNIT,    // radius
	height = 32*FRACUNIT,    // height
	dispoffset = 0,              // display offset
	mass = 100,            // mass
	damage = 1,              // damage
	activesound = sfx_s3kc0s,     // activesound
	flags = MF_SHOOTABLE|MF_MISSILE|MF_DONTENCOREMAP  // flags
}
local rockets = {MT_LAUNCHERROCKET, MT_VERTLROCKET}

for _,i in ipairs(rockets)
	addHook("MobjSpawn", function(mo)
		mo.scale = (3*mapobjectscale)/2
	end, i)
end

local function getLauncher(p)
	xItemLib.func.xItem_setPlayerItemCooldown(p, TICRATE)
	S_StartSound(p.mo, sfx_rlget, p)
end

local function useLauncher(p,cmd)
	if not p.xItemData.xItem_itemSlotLockedTimer
		local bullet = RS_ShootBullet(p, true, MT_LAUNCHERROCKET, 0)
		K_PlayAttackTaunt(p.mo)
		p.kartstuff[k_itemamount] = max($ - 1, 0)
		xItemLib.func.xItem_setPlayerItemCooldown(p, (3*TICRATE)/2)
	end
end

addHook("MobjDamage", function(mo, inf, s)
	if not (mo and mo.valid) then return end
	if not (inf and inf.valid) then return end
	
	local p = mo.player

	if inf.type == MT_LAUNCHERROCKET then
		K_PlayHitEmSound(s)
		K_ExplodePlayer(mo.player, inf, inf.target)
		
		if G_BattleGametype() then
			if K_IsPlayerWanted(mo.player) then
				K_SpawnBattlePoints(s.player, nil, 3)
				s.player.marescore = $ + 3
			else
				K_SpawnBattlePoints(s.player, nil, 1)
				s.player.marescore = $ + 1
			end
		end
		
		if (battleplus and G_BattleGametype() and battleplus.gamemodes[2]) then
			local rings = battleplus.ringrunners.CalculateRingsOnPlayer(mo.player, 15)
			battleplus.ringrunners.GiveRings(mo.player, -rings)
			if (mo and mo.valid)	-- justified paranoia
				S_StartSound(mo, sfx_8blose)
			end
			battleplus.ringrunners.RingBurst(mo, rings, {-5, 5}, {-5, 5}, {20, 24})
		end
		if (hitfeed) then
			local c = s.player.skincolor
			local s = {{s.player.name, s.hf_overridenamecolor or s.player.skincolor}, {skins[s.skin].facemmap, s.player.skincolor}, s.player}
			local v = {{p.name, mo.hf_overridenamecolor or p.skincolor}, {skins[mo.skin].facemmap, p.skincolor}, p}
			local icon = "K_HMRCKT"
			if (s[3] == v[3]) then s = nil end	-- source == victim, remove source
			
			HF_SendHitMessage(s, v, icon, c)
		end
		return true
	end
end, MT_PLAYER)

local function HeavyGunner(p, i)
	return xItemLib.xItemNamespaces["XITEM_ROCKETLAUNCHER"], 2
end


if xItemLib then
	xItemLib.func.addItem{
	"XITEM_ROCKETLAUNCHER", //Namespace
	"Rocket Launcher", //Friendly Name
	"K_ITRCKT", //HUD Graphic
	"K_ISRCKT", //HUD Small Graphic
	XIF_POWERITEM|XIF_COOLDOWNONSTART|XIF_COLPATCH2PLAYER, //Flags
	{ 0, 0, 0, 1, 2, 0, 0, 0, 0, 0 }, //Race Odds
	{2, 1}, //Battle Odds
	getLauncher, //Get Item Function
	useLauncher, //Use Item Function
	false, //HUD Item Function
	false, //Itemodds overrides
	HeavyGunner, //Result Item Function
	{0, {SPR_XRCK, A}}, //Dropped Item Frames
	false, //Show in the item roulette
	nil,
	nil}
end