-- Le Boomstick, by RetroStation

local TICRATE = TICRATE
local FRACUNIT = FRACUNIT
local ANG1 = ANG1
local k_itemamount = k_itemamount
local k_spinouttimer = k_spinouttimer

--item flags
local XIF_POWERITEM = 1 --is power item (affects final odds)
local XIF_COOLDOWNONSTART = 2 --can't be obtained on start cooldown
local XIF_UNIQUE = 4 --only one can exist in anyone's slot
local XIF_LOCKONUSE = 8 --locks the item slot when the item is used, slot must be unlocked manually by setting player.xItemData.xItem_itemSlotLocked to false
local XIF_COOLDOWNINDIRECT = 16 --checks if indirectitemcooldown is 0
local XIF_COLPATCH2PLAYER = 32 --map hud patch colour to player prefcolor
local XIF_ICONFORAMT = 64 --item icon changes depending on the item amount (animation frames become amount frames)

freeslot(
	"sfx_shtgn",
	"sfx_shtget",
	"MT_SHOTGUNPELLET",
	"S_SHOTGUNPELLET",
	"SPR_XSHT"
)

--Thank you Snu for fantastic reference code!
states[S_SHOTGUNPELLET] = {SPR_NULL, A, -1, nil, 0, 0, S_SHOTGUNPELLET}
mobjinfo[MT_SHOTGUNPELLET] = {
	spawnstate = S_SHOTGUNPELLET,
	spawnhealth = 7,
	seesound = sfx_bnce2,
	deathstate = S_SPRK1,
	deathsound = sfx_turhit,
	speed = 240*FRACUNIT,
	radius = 36*FRACUNIT,
	height = 32*FRACUNIT,
	flags = MF_NOGRAVITY|MF_SHOOTABLE|MF_MISSILE
}

local function PumpAction(p, i)
	return xItemLib.xItemNamespaces["XITEM_SHOTGUN"], 2
end

local function getShotty(p)
	S_StartSound(p.mo, sfx_shtget, p)
	xItemLib.func.xItem_setPlayerItemCooldown(p, TICRATE/2)
end
--addHook("MobjSpawn", function(mo)
	--mo.criticalhit = 3
--end, MT_SHOTGUNPELLET)

local function useShotty(p,cmd)
	local bullet = RS_ShootBullet(p, true, MT_SHOTGUNPELLET, 0)
	p.kartstuff[k_itemamount] = $-1
	K_PlayAttackTaunt(p.mo)
	S_StartSound(p.mo, sfx_shtgn)
	xItemLib.func.xItem_setPlayerItemCooldown(p, TICRATE/2)

	--xItemLib.func.xItem_setPlayerItemCooldown(p, TICRATE)
end

local function pelletHandler(mo)
	if not (mo and mo.valid) then return end
	
	if mo.criticalhit then
		mo.criticalhit = max($ - 1, 0)
		--print("Crit Active")
	end
end

local function criticalVictim (p)
	if p.kartstuff[k_spinouttimer] then
		if p.boomstick then
			--print("F U C K E D")
			if not p.mo.criticalshot then
				p.mo.momz = FixedMul($ + (12*mapobjectscale), sin(ANGLE_45))
				p.mo.criticalshot = true
				S_StartSound(p.mo, sfx_toada)
			end

			if not P_IsObjectOnGround(p.mo) then
				p.kartstuff[k_spinouttimer] = $+1
			end
				
			if p.kartstuff[k_spinouttimer] == 1 then
				p.boomstick = false
				if p.mo.criticalshot then
					p.mo.criticalshot = nil
				end
			end
		end
	end
end

local function criticalHandler(p, inf, s)
	if not (p.mo and p.mo.valid) or p.spectator then return end
	
	if (inf and inf.valid)
		if inf.criticalhit then
			p.boomstick = true
		end
	end
end

--addHook("PlayerSpin", criticalHandler)
addHook("MobjThinker", pelletHandler, MT_SHOTGUNPELLET)
addHook("PlayerThink", criticalVictim)

addHook("MobjDamage", function(mo, inf, s)
	if not ((mo and mo.valid) and mo.player) then return end
	local p = mo.player
	
	if not ((inf and inf.valid) and inf.type == MT_SHOTGUNPELLET) then return end
	
	-- get base knockback
	local pushx = FixedMul(FRACUNIT/7, inf.momx)
	local pushy = FixedMul(FRACUNIT/7, inf.momy)

	-- these will stack up close for a devastating effect, since multiple can hit
	pushx = FixedMul(FRACUNIT*9/20, inf.momx)
	pushy = FixedMul(FRACUNIT*9/20, inf.momy)
	
	if not (p.powers[pw_flashing]) then
		mo.momx = $ + pushx
		mo.momy = $ + pushy
	end
	S_StartSound(mo, sfx_s3k7b)

	-- spinout the player manually so we can wipeout the player instead of spinout
	if (p.kartstuff[k_invincibilitytimer] > 0)
	or (p.kartstuff[k_growshrinktimer] > 0	or (HugeQuest and p.hugequest.huge > 0))
	or (p.powers[pw_flashing])
		-- insta shield them instead!
		K_DoInstashield(p)
	else
		if ((s and s.valid) and s.player)
			K_PlayHitEmSound(s)
		end
		K_SpinPlayer(p, inf, 1, inf.target)
		if inf.criticalhit then
			p.boomstick = true
		end
		P_PlayRinglossSound(mo)
		
		if G_BattleGametype() then
			if K_IsPlayerWanted(mo.player) then
				K_SpawnBattlePoints(s.player, nil, 3)
				s.player.marescore = $ + 3
			else
				K_SpawnBattlePoints(s.player, nil, 1)
				s.player.marescore = $ + 1
			end
		end

		if (hitfeed) then
			local c = s.player.skincolor
			local s = {{s.player.name, s.hf_overridenamecolor or s.player.skincolor}, {skins[s.skin].facemmap, s.player.skincolor}, s.player}
			local v = {{p.name, mo.hf_overridenamecolor or p.skincolor}, {skins[mo.skin].facemmap, p.skincolor}, p}
			local icon = "K_HMXSHT"
			if (s[3] == v[3]) then s = nil end	-- source == victim, remove source
			
			HF_SendHitMessage(s, v, icon, c)
		end
	end
	
	return true
end, MT_PLAYER)

if xItemLib then
	xItemLib.func.addItem{
	"XITEM_SHOTGUN", //Namespace
	"Shotgun", //Friendly Name
	"K_ITXSHT", //HUD Graphic
	"K_ISXSHT", //HUD Small Graphic
	XIF_POWERITEM, //Flags
	{ 0, 0, 0, 2, 1, 0, 0, 0, 0, 0 }, //Race Odds
	{2, 1}, //Battle Odds
	getShotty, //Get Item Function
	useShotty, //Use Item Function
	false, //HUD Item Function
	false, //Itemodds overrides
	PumpAction, //Result Item Function
	{0, {SPR_XSHT,A}}, //Dropped Item Frames
	true, //Show in the item roulette
	nil,
	nil}
end